Package: rainette
Type: Package
Title: The Reinert Method for Textual Data Clustering
Version: 0.3.3
Date: 2026-05-04
Authors@R: c(person("Julien", "Barnier", email="julien.barnier@cnrs.fr",
    role=c("aut","cre")), person("Florian", "Privé",
    email="florian.prive.21@gmail.com", role="ctb"))
Maintainer: Julien Barnier <julien.barnier@cnrs.fr>
Description: An R implementation of the Reinert text clustering method. For more
    details about the algorithm see the included vignettes or Reinert (1990)
    <doi:10.1177/075910639002600103>.
License: GPL (>= 3)
VignetteBuilder: knitr
URL: https://juba.github.io/rainette/
BugReports: https://github.com/juba/rainette/issues
Encoding: UTF-8
Depends: R (>= 3.6.0)
Imports: dplyr (>= 1.1.0), tidyr, purrr, ggplot2, stringr, quanteda (>=
        2.1), quanteda.textstats, RSpectra, dendextend, ggwordcloud,
        gridExtra, rlang, shiny, miniUI, highr, progressr, Rcpp (>=
        1.0.3)
Suggests: testthat, knitr, rmarkdown, tm, FNN, vdiffr,
        quanteda.textmodels
RoxygenNote: 7.3.2
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-05-04 13:28:24 UTC; julien
Author: Julien Barnier [aut, cre],
  Florian Privé [ctb]
Repository: CRAN
Date/Publication: 2026-05-04 18:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-04 22:10:48 UTC; unix
Archs: rainette.so.dSYM
