# Generated by roxygen2: do not edit by hand

S3method(split_segments,Corpus)
S3method(split_segments,character)
S3method(split_segments,corpus)
S3method(split_segments,tokens)
export(clusters_by_doc_table)
export(cutree)
export(cutree_rainette)
export(cutree_rainette2)
export(docs_by_cluster_table)
export(import_corpus_iramuteq)
export(merge_segments)
export(rainette)
export(rainette2)
export(rainette2_complete_groups)
export(rainette2_explor)
export(rainette2_plot)
export(rainette_explor)
export(rainette_plot)
export(rainette_stats)
export(split_segments)
import(dendextend)
import(dplyr)
import(ggplot2)
import(ggwordcloud)
import(miniUI)
import(quanteda)
import(shiny)
import(stringr)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(purrr,map_chr)
importFrom(purrr,map_int)
importFrom(rlang,.env)
importFrom(rlang,sym)
useDynLib(rainette)
